using System.ComponentModel.DataAnnotations;
using System.Security.Claims;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Todo.Core.Interfaces.Persistence;

namespace Todo.Api.Controllers;

[ApiController]
[Route("/api/todos")]
public class TodosController : ControllerBase
{
    private readonly ITodoRepository _todoRepository;

    public TodosController(ITodoRepository todoRepository)
    {
        _todoRepository = todoRepository;
    }

    [HttpPost]
    [Authorize]
    public async Task<ActionResult<Core.Entities.Todo>> CreateTodo(
        [FromBody] CreateTodoRequest request)
    {
        var userId = User.FindFirstValue("sub") ??
                     throw new InvalidOperationException("Could not get user, something has gone terribly wrong");

        return Ok(
            await _todoRepository.CreateTodoAsync(
                request.Title,
                string.Empty,
                "",
                userId));
    }

    [HttpGet]
    [Authorize]
    public async Task<ActionResult<IEnumerable<Core.Entities.Todo>>> GetTodos()
    {
        return Ok(await _todoRepository.GetTodosAsync());
    }

    [HttpGet("not-done")]
    public async Task<ActionResult<IEnumerable<Core.Entities.Todo>>>
        GetNotDoneTodos()
    {
        return Ok(await _todoRepository.GetNotDoneTodos());
    }

    public record CreateTodoRequest
    {
        [Required]
        public string Title { get; init; }
    }
}